package dk.apaq.crud;

/**
 * Interface for listeners of Crud actions.
 */
public interface CrudListener<IDT, BT extends HasId<IDT>> {

    /**
     * Called before an entity is read.
     */
    void onBeforeEntityRead(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called when an entity is read.
     */
    void onEntityRead(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called before an entity is deleted.
     */
    void onBeforeEntityDelete(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called when an entity is deleted.
     * @param event
     */
    void onEntityDelete(CrudEvent.WithId<IDT, BT> event);

    /**
     * Called before an entity is being persisted.
     */
    void onBeforeEntityUpdate(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called when an entity is persisted.
     * @param event
     */
    void onEntityUpdate(CrudEvent.WithEntity<IDT, BT> event);

    /**
     * Called before an entity is created.
     */
    void onBeforeEntityCreate(CrudEvent<IDT, BT> event);

    /**
     * Called when an entity is created.
     */
    void onEntityCreate(CrudEvent.WithEntity<IDT, BT> event);
}

